/* eslint-disable @typescript-eslint/no-explicit-any */
import { AirLines, Prisma } from '@prisma/client'
import ApiError from '../../error/ApiError'
import { prisma } from '../../shared/prisma'
import httpStatus from 'http-status'
import peginationOption from '../../helper/peginationHelpers'
import { TParams } from './airLines.interface'

//create airline
const createAirIntoDB = async (payload: any) => {
  const isAirExist = await prisma.airLines.findUnique({
    where: {
      airLinesName: payload.airLinesName,
    },
  })

  if (isAirExist) {
    throw new ApiError(httpStatus.FORBIDDEN, 'Airline already exist')
  }

  // const airlineData = {
  //   airLinesName: req.body.airLinesName,
  //   email: req.body.email,
  //   contactNumber: req.body.contactNumber,
  //   emergencyContactNumber: req.body.emergencyContactNumber,
  //   planeQuantity: req.body.planeQuantity,
  //   address: req.body.address,
  //   logo: req.file?.path,
  // }

  const result = await prisma.airLines.create({
    data: payload,
  })
  return result
}

//get all airlines with sorting, searching, limit and pagination
const getAirLinesIntoDB = async (params: TParams, options: any) => {
  const { ...filterData } = params
  const { page, limit, skip, sortBy, sortOrder } = peginationOption(options)
  const andConditions: Prisma.AirLinesWhereInput[] = []

  const searchAbleField = ['airLinesName']

  if (params.searchTerm) {
    andConditions.push({
      OR: searchAbleField.map(fields => ({
        [fields]: {
          contains: params.searchTerm,
          mode: 'insensitive',
        },
      })),
    })
  }

  if (Object.keys(filterData).length > 0) {
    andConditions.push({
      AND: Object.keys(filterData).map(key => ({
        [key]: {
          equals: (filterData as any)[key],
        },
      })),
    })
  }

  const whereCondition: Prisma.AirLinesWhereInput = { AND: andConditions }
  const result = await prisma.airLines.findMany({
    where: whereCondition,
    skip: skip,
    take: limit,
    orderBy:
      sortBy && sortOrder
        ? {
            [sortBy]: sortOrder,
          }
        : {
            createdAt: 'desc',
          },
    include: {
      planes: true,
    },
  })

  const totalCount = await prisma.airLines.count({
    where: whereCondition,
  })

  return {
    meta: {
      limit,
      page,
      total: totalCount,
      totalPages: Math.ceil(totalCount / limit),
    },
    data: result,
  }
}

//get single airline by id
const getAirLinesByIdIntoDB = async (id: string) => {
  const result = await prisma.airLines.findUnique({
    where: {
      id,
    },
    include: {
      planes: true,
    },
  })

  if (!result) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Airline not found')
  }

  return result
}

//update airline info
const updateAirLinesInfoIntoDB = async (
  id: string,
  payload: Partial<AirLines>,
) => {
  const {
    email,
    contactNumber,
    emergencyContactNumber,
    planeQuantity,
    address,
  } = payload
  const findAirLines = await prisma.airLines.findUnique({
    where: {
      id,
    },
  })

  if (!findAirLines) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Airline not found')
  }

  const result = await prisma.airLines.update({
    where: {
      id,
    },
    data: {
      email,
      contactNumber,
      emergencyContactNumber,
      planeQuantity,
      address,
    },
  })

  return result
}

//soft delete airline
const softDeleteAirLinesIntoDB = async (id: string) => {
  const findAirLines = await prisma.airLines.findUnique({
    where: {
      id,
    },
  })

  if (!findAirLines) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Airline not found')
  }

  const result = await prisma.$transaction(async transactionClient => {
    await transactionClient.planeDetails.updateMany({
      where: {
        airLinesId: id,
      },
      data: {
        isDeleted: true,
      },
    })

    const airLinesData = await transactionClient.airLines.update({
      where: {
        id,
      },
      data: {
        isDeleted: true,
      },
    })
    return airLinesData
  })

  return result
}

export const AirService = {
  createAirIntoDB,
  getAirLinesIntoDB,
  getAirLinesByIdIntoDB,
  updateAirLinesInfoIntoDB,
  softDeleteAirLinesIntoDB,
}
