/* eslint-disable no-unused-vars */
/* eslint-disable @typescript-eslint/no-unused-vars */
import { addHours, isAfter } from 'date-fns'
import { prisma } from '../../shared/prisma'
import { TAuth, TRegister } from './auth.interface'
import bcrypt from 'bcryptjs'
import { UserRoles, UserStatus } from '@prisma/client'
import config from '../../config'
import { jwtHelper } from '../../helper/jwtHelper'
import ApiError from '../../error/ApiError'
import httpStatus from 'http-status'
import { JwtPayload } from 'jsonwebtoken'
import emailSender from './emailSender'

//register
const register = async (payload: TRegister) => {
  const userExists = await prisma.user.findUnique({
    where: {
      contactNumber: payload.contactNumber,
      status: UserStatus.ACTIVE,
      isDeleted: false,
    },
  })

  if (userExists) {
    throw new ApiError(
      httpStatus.BAD_REQUEST,
      'Already have an account. Use Different Email or Contact Number',
    )
  }

  const hasedPasseword = await bcrypt.hash(
    payload.password,
    Number(config.salt_round),
  )

  const userData = {
    email: payload.email,
    contactNumber: payload.contactNumber,
    password: hasedPasseword,
    role: UserRoles.PASSENGER,
  }

  const passengerData = {
    fullName: payload.fullName,
    email: payload.email,
    contactNumber: payload.contactNumber,
    emergencyContactNumber: payload.emergencyContactNumber,
    dateOfBirth: payload.dateOfBirth,
    gender: payload.gender,
  }

  const result = await prisma.$transaction(async transactionClient => {
    await transactionClient.user.create({
      data: userData,
    })

    const passenger = await transactionClient.passenger.create({
      data: passengerData,
    })
    return passenger
  })

  return result
}

//user login
const login = async (payload: TAuth) => {
  const user = await prisma.user.findUnique({
    where: {
      contactNumber: payload.contactNumber,
    },
    include: {
      busComapanyEmployee: {
        select: {
          companyName: true,
        },
      },
      passenger: true,
      companyOwner: {
        select: {
          companyName: true,
        },
      },
    },
  })

  if (!user) {
    throw new ApiError(httpStatus.NOT_FOUND, 'User  not found')
  }

  const maxAttempts = 5
  const lockOutDuration = 2

  const now = new Date()
  if (user.lockOutUntil && isAfter(now, user.lockOutUntil)) {
    await prisma.user.update({
      where: { id: user.id },
      data: { failedAttempts: 0, lockOutUntil: null },
    })
  } else if (user.lockOutUntil) {
    throw new ApiError(
      httpStatus.BAD_REQUEST,
      `Your account is locked, try again after ${user.lockOutUntil}`,
    )
  }

  const isMatch = await bcrypt.compare(payload.password, user.password)
  const failedAttempts = isMatch ? 0 : user.failedAttempts + 1

  if (!isMatch) {
    if (failedAttempts >= maxAttempts) {
      const lockOutUntil = addHours(now, lockOutDuration)
      await prisma.user.update({
        where: { id: user.id },
        data: { failedAttempts, lockOutUntil },
      })
      throw new ApiError(
        httpStatus.BAD_REQUEST,
        `Your account is locked, try again after ${lockOutUntil}`,
      )
    }

    await prisma.user.update({
      where: { id: user.id },
      data: { failedAttempts },
    })
    throw new ApiError(httpStatus.BAD_REQUEST, 'Invalid password')
  }

  await prisma.user.update({
    where: { id: user.id },
    data: { failedAttempts: 0, lockOutUntil: null },
  })

  let jwtPayload

  if (user.role === UserRoles.OWNER) {
    jwtPayload = {
      contactNumber: user.contactNumber,
      role: user.role,
      companyName: user.companyOwner?.companyName,
    }
  } else if (user.role === UserRoles.ADMIN) {
    jwtPayload = {
      contactNumber: user.contactNumber,
      role: user.role,
      companyName: user.busComapanyEmployee?.companyName,
    }
  } else {
    jwtPayload = {
      contactNumber: user.contactNumber,
      role: user.role,
    }
  }

  const accessToken = jwtHelper.tokenGenerate(
    jwtPayload,
    config.access_token_sceret as string,
    config.access_token_expires_in as string,
  )

  const refreshToken = jwtHelper.tokenGenerate(
    jwtPayload,
    config.refresh_token_sceret as string,
    config.refresh_token_expires_in as string,
  )

  return {
    accessToken,
    refreshToken,
  }
}

const createRefreshToken = async (token: string) => {
  let decoded
  try {
    decoded = jwtHelper.verifyToken(
      token,
      config.refresh_token_sceret as string,
    ) as JwtPayload
  } catch (error) {
    throw new ApiError(httpStatus.FORBIDDEN, 'Invalid refresh token')
  }

  const userFind = await prisma.user.findUnique({
    where: {
      contactNumber: decoded.contactNumber,
      isDeleted: false,
      status: UserStatus.ACTIVE,
    },
    include: {
      busComapanyEmployee: {
        select: {
          companyName: true,
        },
      },
      passenger: true,
      companyOwner: {
        select: {
          companyName: true,
        },
      },
    },
  })

  if (!userFind) {
    throw new ApiError(httpStatus.NOT_FOUND, 'User not found')
  }

  let jwtPayload

  if (userFind.role === UserRoles.OWNER) {
    jwtPayload = {
      contactNumber: userFind.contactNumber,
      role: userFind.role,
      companyName: userFind.companyOwner?.companyName,
    }
  } else if (userFind.role === UserRoles.ADMIN) {
    jwtPayload = {
      contactNumber: userFind.contactNumber,
      role: userFind.role,
      companyName: userFind.busComapanyEmployee?.companyName,
    }
  } else {
    jwtPayload = {
      contactNumber: userFind.contactNumber,
      role: userFind.role,
    }
  }

  const accessToken = jwtHelper.tokenGenerate(
    jwtPayload,
    config.access_token_sceret as string,
    config.access_token_expires_in as string,
  )

  return accessToken
}

//forgetPassword
const forgetPassword = async (payload: { email: string }) => {
  const user = await prisma.user.findUnique({
    where: {
      email: payload.email,
    },
    select: {
      email: true,
      contactNumber: true,
    },
  })

  if (!user) {
    throw new ApiError(httpStatus.NOT_FOUND, 'User not found')
  }

  const token = jwtHelper.tokenGenerate(
    { email: user.email },
    config.reset_password_secret as string,
    config.reset_password_expires_in as string,
  )

  const resetLink =
    config.reset_password_reset_password_url +
    `?email=${user.email}&token=${token}`

  await emailSender(
    user.email as string,
    `<!DOCTYPE html>
<html>
<body style="font-family: Arial, sans-serif; background-color: #f4f4f4; margin: 0; padding: 0;">
  <div style="max-width: 600px; margin: 20px auto; background-color: #ffffff; padding: 20px; border-radius: 8px; box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);">
    <div style="text-align: center; background-color: #1990cb; padding: 10px 0; border-radius: 8px 8px 0 0; color: white;">
      <h1 style="margin: 0; font-size: 24px;">Reset Your Password</h1>
    </div>
    <div style="padding: 20px; text-align: center; color: #333333;">
      <h1 style="font-size: 24px; margin-bottom: 10px;">Hi there,</h1>
      <p style="font-size: 16px; margin-bottom: 20px; line-height: 1.5;">
        You recently requested to reset your password. Click the button below to reset it.
      </p>
      <a href="${resetLink}" style="display: inline-block; text-decoration: none; background-color: #1990cb; color: white; padding: 10px 20px; border-radius: 5px; font-size: 16px;">Reset Password</a>
    </div>
    <div style="text-align: center; margin-top: 20px; font-size: 12px; color: #666666;">
      <p style="margin: 0;">If you have any issues, please contact our support team.</p>
      <p style="margin: 0;">&copy; ${new Date().getFullYear()} E-Ticket DhakaBoss. All rights reserved.</p>
    </div>
  </div>
</body>
</html>
      `,
  )
}

const resetPasswordInto = async (
  token: string,
  payload: { password: string },
) => {
  const decodedToken = jwtHelper.verifyToken(
    token,
    config.reset_password_secret as string,
  ) as JwtPayload

  if (!decodedToken) {
    throw new ApiError(httpStatus.UNAUTHORIZED, 'Invalid token')
  }

  const userFind = await prisma.user.findUnique({
    where: {
      email: decodedToken.email,
    },
  })

  if (!userFind) {
    throw new ApiError(httpStatus.NOT_FOUND, 'User not found')
  }

  const hashedPassword = await bcrypt.hash(
    payload.password,
    Number(config.salt_round),
  )

  await prisma.user.update({
    where: {
      email: decodedToken.email,
    },
    data: {
      password: hashedPassword,
    },
  })

  return {
    message: 'Password reset successfully',
  }
}

export const AuthService = {
  login,
  register,
  createRefreshToken,
  forgetPassword,
  resetPasswordInto,
}
