/* eslint-disable @typescript-eslint/no-explicit-any */
import { Request } from 'express'
import { prisma } from '../../../shared/prisma'
import ApiError from '../../../error/ApiError'
import httpStatus from 'http-status'

const createBusDetailsIntoDB = async (req: Request) => {
  const { companyName } = req.user

  const payload = req.body

  const companyFind = await prisma.busCompany.findFirst({
    where: {
      companyName: companyName,
    },
  })

  if (!companyFind) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Company not found')
  }

  const result = await prisma.busDetails.create({
    data: {
      busCompanyId: companyFind.id,
      ...payload,
    },
  })
  return result
}

const getAllBusFromDB = async (req: Request, ownerId: string) => {
  const { companyName } = req.user

  const result = await prisma.busDetails.findMany({
    where: {
      OR: [
        { busCompany: { companyName: companyName } },
        { busCompany: { busOwnerName: ownerId } },
      ],
    },
    include: {
      busCompany: true,
      busCounter: true,
      busBoardingPoint: true,
      busDroppingPoint: true,
    },
  })

  return result
}

const getSpicificBusDetails = async (id: string) => {
  const result = await prisma.busDetails.findUnique({
    where: {
      id: id,
    },
    include: {
      busSchedule: true,
      busCounter: true,
    },
  })
  return result
}

const getAllBusForSearch = async (query: any) => {
  const { startPoint, endPoint, departureDate } = query

  const result = await prisma.busDetails.findMany({
    where: {
      OR: [
        {
          busSchedule: {
            some: {
              startPoint: {
                contains: startPoint,
                mode: 'insensitive',
              },
              endPoint: {
                contains: endPoint,
                mode: 'insensitive',
              },
              departureDate: {
                equals: departureDate,
              },
            },
          },
        },
        {
          busRoute: {
            some: {
              stoppageLocation: {
                contains: startPoint,
                mode: 'insensitive',
              },
            },
          },
        },
      ],
    },
    include: {
      busSchedule: {
        where: {
          endPoint: {
            contains: endPoint,
            mode: 'insensitive',
          },
          departureDate: {
            contains: departureDate,
          },
        },
      },
      busRoute: true,
      busBoardingPoint: true,
      busDroppingPoint: true,
      busCompany: true,
    },
  })

  return result
}

const getAllBusForSuperAdmin = async () => {
  const result = await prisma.busDetails.findMany({})
  return result
}

export const BusDetailsService = {
  createBusDetailsIntoDB,
  getAllBusFromDB,
  getSpicificBusDetails,
  getAllBusForSearch,
  getAllBusForSuperAdmin,
}
