/* eslint-disable @typescript-eslint/no-explicit-any */
import { Flight, Prisma } from '@prisma/client'
import { prisma } from '../../shared/prisma'
import ApiError from '../../error/ApiError'
import httpStatus from 'http-status'

const createFlightIntoDB = async (payload: Flight) => {
  const isPlaneExist = await prisma.planeDetails.findUnique({
    where: {
      id: payload.planeId,
    },
  })

  if (!isPlaneExist) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Plane not found')
  }
  const result = await prisma.flight.create({
    data: payload,
  })
  return result
}

const getFlightFromDB = async (params: any) => {
  console.log('Params received from frontend:', params)

  const {
    ticketType,
    fromAirportCode,
    toAirportCode,
    journeyDate,
    returnDate,
    airlinesId,
    sortBy,
  } = params

  if (!ticketType || !fromAirportCode || !toAirportCode || !journeyDate) {
    throw new Error(
      'Required fields: ticketType, fromAirportCode, toAirportCode, and journeyDate.',
    )
  }

  // Define sorting order based on sortBy parameter
  const getSortOrder = (): Prisma.FlightOrderByWithRelationInput => {
    if (!sortBy) return { createdAt: Prisma.SortOrder.desc }
    return sortBy === 'price-low-high'
      ? { price: Prisma.SortOrder.asc }
      : { price: Prisma.SortOrder.desc }
  }

  const airlineFilter: Prisma.FlightWhereInput = airlinesId
    ? { plane: { airLines: { id: { equals: airlinesId } } } }
    : {}

  if (ticketType === 'oneWay') {
    // One-way flights code remains the same
    const oneWayFlights = await prisma.flight.findMany({
      where: {
        AND: [
          {
            fromAirport: {
              airportCode: { equals: fromAirportCode, mode: 'insensitive' },
            },
          },
          {
            toAirport: {
              airportCode: { equals: toAirportCode, mode: 'insensitive' },
            },
          },
          { schedules: { some: { journeyDates: { has: journeyDate } } } },
          airlineFilter,
        ],
      },
      orderBy: getSortOrder(),
      include: {
        plane: {
          select: {
            id: true,
            planeName: true,
            planeId: true,
            flightType: true,
            capacity: true,
            isDeleted: true,
            airLines: true,
          },
        },
        schedules: {
          select: {
            departureTime: true,
            arrivalTime: true,
            duration: true,
          },
        },
        fromAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
        toAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
      },
    })
    return oneWayFlights
  }

  if (ticketType === 'roundTrip' && journeyDate && returnDate) {
    // Get all outgoing and return flights first without sorting
    const outgoingFlights = await prisma.flight.findMany({
      where: {
        AND: [
          {
            fromAirport: {
              airportCode: { equals: fromAirportCode, mode: 'insensitive' },
            },
          },
          {
            toAirport: {
              airportCode: { equals: toAirportCode, mode: 'insensitive' },
            },
          },
          { schedules: { some: { journeyDates: { has: journeyDate } } } },
          airlineFilter,
        ],
      },
      include: {
        plane: {
          select: {
            id: true,
            planeName: true,
            planeId: true,
            flightType: true,
            capacity: true,
            isDeleted: true,
            airLines: true,
          },
        },
        schedules: {
          select: {
            departureTime: true,
            arrivalTime: true,
            duration: true,
          },
        },
        fromAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
        toAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
      },
    })

    const returnFlights = await prisma.flight.findMany({
      where: {
        AND: [
          {
            fromAirport: {
              airportCode: { equals: toAirportCode, mode: 'insensitive' },
            },
          },
          {
            toAirport: {
              airportCode: { equals: fromAirportCode, mode: 'insensitive' },
            },
          },
          { schedules: { some: { journeyDates: { has: returnDate } } } },
          airlineFilter,
        ],
      },
      include: {
        plane: {
          select: {
            id: true,
            planeName: true,
            planeId: true,
            flightType: true,
            capacity: true,
            isDeleted: true,
            airLines: true,
          },
        },
        schedules: {
          select: {
            departureTime: true,
            arrivalTime: true,
            duration: true,
          },
        },
        fromAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
        toAirport: {
          select: {
            id: true,
            airportCode: true,
            airPortName: true,
            city: true,
            country: true,
          },
        },
      },
    })

    // Create paired flights with total price
    const pairedFlightsWithPrice = outgoingFlights.flatMap(outFlight =>
      returnFlights
        .filter(
          retFlight =>
            retFlight.plane.airLines.id === outFlight.plane.airLines.id,
        )
        .map(retFlight => ({
          outgoingFlight: outFlight,
          returnFlight: retFlight,
          totalPrice: Number(outFlight.price) + Number(retFlight.price),
        })),
    )

    // Sort the paired flights based on total price
    const sortedPairedFlights = pairedFlightsWithPrice.sort((a, b) => {
      if (sortBy === 'price-low-high') {
        return a.totalPrice - b.totalPrice
      } else if (sortBy === 'price-high-low') {
        return b.totalPrice - a.totalPrice
      }
      // Default sorting by creation date if no sort specified
      return (
        new Date(b.outgoingFlight.createdAt).getTime() -
        new Date(a.outgoingFlight.createdAt).getTime()
      )
    })

    return sortedPairedFlights
  }

  throw new Error('Invalid parameters or unsupported ticketType.')
}

const getSingleFlightFromDB = async (
  outGoingFlighId: string,
  returnFlighId?: string,
) => {
  // Initialize an array to store the flights
  const flightIds = [outGoingFlighId]

  // If returnFlighId is provided, add it to the array
  if (returnFlighId) {
    flightIds.push(returnFlighId)
  }

  // Fetch flights using `findMany`
  const flights = await prisma.flight.findMany({
    where: {
      id: {
        in: flightIds, // This will find both if both IDs are provided or just the outgoing flight if returnFlighId is not available
      },
      isDeleted: false,
    },
    include: {
      plane: {
        select: {
          id: true,
          planeName: true,
          planeId: true,
          flightType: true,
          capacity: true,
          isDeleted: true,
          airLines: true,
        },
      },
      schedules: {
        select: {
          departureTime: true,
          arrivalTime: true,
          duration: true,
        },
      },
      fromAirport: {
        select: {
          id: true,
          airportCode: true,
          airPortName: true,
          city: true,
          country: true,
        },
      },
      toAirport: {
        select: {
          id: true,
          airportCode: true,
          airPortName: true,
          city: true,
          country: true,
        },
      },
    },
  })

  // Return flights as an array
  return flights
}

export const FlightService = {
  createFlightIntoDB,
  getFlightFromDB,
  getSingleFlightFromDB,
}

// flatMap:

//     Loops through each outgoing flight.
//     For each outgoing flight, it finds matching return flights (from the same airline) using .filter().
//     It then combines each pair of outgoing and return flights into a single object with a totalPrice.
//     flatMap flattens the pairs into one array, instead of creating a nested array.
