import { Request } from 'express'
import { prisma } from '../../shared/prisma'
import ApiError from '../../error/ApiError'
import httpStatus from 'http-status'

const getPassengerForSpeicificBus = async (req: Request) => {
  const { contactNumber } = req.params

  const result = await prisma.passenger.findFirst({
    where: {
      contactNumber: contactNumber,
      isDeleted: false,
    },
  })

  if (!result) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Passenger not found')
  }

  return result
}

const updatePassengerInfoIntoDB = async (req: Request) => {
  const { contactNumber } = req.user
  const file = req.file
  const payload = req.body

  const result = await prisma.passenger.update({
    where: {
      contactNumber: contactNumber,
    },
    data: {
      ...payload,
      profileImage: file?.path,
    },
  })

  return result
}

const getPassengerForSpicificUser = async (req: Request) => {
  const { contactNumber } = req.user

  const result = await prisma.passengerInfoForBus.findMany({
    where: {
      contactNumber: contactNumber,
    },
  })

  if (!result) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Passenger not found')
  }

  return result
}

const calculatedInsight = async (req: Request) => {
  const { contactNumber } = req.user

  const isExistPassenger = await prisma.passenger.findFirst({
    where: {
      contactNumber: contactNumber,
    },
  })

  if (!isExistPassenger) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Passenger not found')
  }

  const totalCostForPassenger = await prisma.passengerInfoForBus.aggregate({
    _sum: {
      ticketPrice: true,
    },
    where: {
      contactNumber: contactNumber,
    },
  })

  const totalTicketCount = await prisma.passengerInfoForBus.count({
    where: {
      contactNumber: contactNumber,
    },
  })

  const calculatedTicket = totalCostForPassenger._sum.ticketPrice

  const monthlyData = await prisma.passengerInfoForBus.groupBy({
    by: ['depertureDate'],
    _sum: {
      ticketPrice: true,
    },
    _count: {
      id: true,
    },
    where: {
      contactNumber: contactNumber,
    },
  })

  const formattedData = monthlyData.map(data => ({
    flights: data._count.id,
    spent: data._sum.ticketPrice || 0,
  }))

  return {
    calculatedTicket,
    totalTicketCount,
    formattedData,
  }
}

export const PassengerService = {
  getPassengerForSpeicificBus,
  updatePassengerInfoIntoDB,
  getPassengerForSpicificUser,
  calculatedInsight,
}
