/* eslint-disable @typescript-eslint/no-explicit-any */
import bcrypt from 'bcryptjs'
import config from '../../config'
import { prisma } from '../../shared/prisma'
import { UserRoles, UserStatus } from '@prisma/client'
import { Request } from 'express'
import ApiError from '../../error/ApiError'
import httpStatus from 'http-status'
import { IAuthUser } from '../../interface/common'

//create bus company owner
const createOwnerIntoDB = async (req: Request) => {
  const payload = req.body
  const file = req.file

  const hasedPasseword = await bcrypt.hash(
    payload.password,
    Number(config.salt_round),
  )

  const userData = {
    email: payload.companyOwner.email,
    contactNumber: payload.companyOwner.contactNumber,
    password: hasedPasseword,
    role: UserRoles.OWNER,
  }

  const companyOwnerData = {
    ...payload.companyOwner,
    profileImage: file?.path,
  }

  const result = await prisma.$transaction(async transactionClient => {
    await transactionClient.user.create({
      data: userData,
    })
    const ownerData = await transactionClient.busCompanyOwner.create({
      data: companyOwnerData,
    })
    return ownerData
  })

  return result
}

//!create bus company employee -- deprecated
const createBusCompanyEmployeeIntoDB = async (
  payload: any,
  companyName: string,
) => {
  const hasedPasseword = await bcrypt.hash(
    payload.password,
    Number(config.salt_round),
  )
  const userData = {
    email: payload.admin.email,
    contactNumber: payload.admin.contactNumber,
    password: hasedPasseword,
    role: payload.admin.designation,
  }

  const adminData = {
    companyName: companyName,
    ...payload.admin,
  }

  const result = await prisma.$transaction(async transactionClient => {
    await transactionClient.user.create({
      data: userData,
    })

    const result = await transactionClient.busComapnyEmployee.create({
      data: adminData,
    })

    return result
  })

  return result
}

//create bus counter employee
const createBusCounterEmployeeIntoDB = async (req: Request) => {
  const file = req?.file
  const payload = req.body

  const { contactNumber } = req.user

  const findCompany = await prisma.busCompanyOwner.findUnique({
    where: {
      contactNumber: contactNumber,
    },
    include: {
      busCompany: {
        include: {
          busCounter: true,
        },
      },
    },
  })

  if (!findCompany) {
    throw new ApiError(httpStatus.UNAUTHORIZED, 'Company owner not found')
  }

  const hasedPasseword = await bcrypt.hash(
    payload.password,
    Number(config.salt_round),
  )

  const userData = {
    email: payload.counterEmployee.email,
    contactNumber: payload.counterEmployee.contactNumber,
    password: hasedPasseword,
    role: UserRoles.ADMIN,
  }

  const employeeData = {
    companyId: findCompany?.busCompany?.id,
    profileImage: file?.path,
    designation: UserRoles.ADMIN,
    ...payload.counterEmployee,
  }

  const result = await prisma.$transaction(async transactionClient => {
    await transactionClient.user.create({
      data: userData,
    })

    const result = await transactionClient.busCounterEmployee.create({
      data: employeeData,
    })

    return result
  })

  return result
}

//getAll employee for a bus company
const getAllEmployeeForSpicificBusCompany = async (req: Request) => {
  const { companyName } = req.user
  if (!companyName) {
    throw new ApiError(httpStatus.UNAUTHORIZED, 'Company name not found')
  }

  const result = await prisma.busCompany.findUnique({
    where: {
      companyName: companyName,
    },
    select: {
      id: true,
    },
  })

  if (!result) {
    throw new ApiError(httpStatus.BAD_REQUEST, 'Company not found')
  }

  const employeeList = await prisma.busCounterEmployee.findMany({
    where: {
      companyId: result?.id,
      isDeleted: false,
    },
  })

  return employeeList
}

//getSpicificUser
const getSpicificUser = async (payload: string) => {
  const result = await prisma.busCompanyOwner.findUnique({
    where: {
      contactNumber: payload,
    },
  })

  return result
}

const getAllBusOwner = async () => {
  const result = await prisma.busCompanyOwner.findMany()
  return result
}

const getAllPassenger = async () => {
  const result = await prisma.passenger.findMany()
  return result
}

const getMe = async (req: Request) => {
  const { contactNumber } = req.user

  const userInfo = await prisma.user.findUniqueOrThrow({
    where: {
      contactNumber: contactNumber,
    },
    select: {
      id: true,
      email: true,
      contactNumber: true,
      role: true,
      status: true,
    },
  })

  let profileInfo

  if (userInfo.role === UserRoles.SUPER_ADMIN) {
    profileInfo = await prisma.superAdmin.findUnique({
      where: {
        contactNumber: userInfo.contactNumber,
      },
    })
  } else if (userInfo.role === UserRoles.OWNER) {
    profileInfo = await prisma.busCompanyOwner.findUnique({
      where: {
        contactNumber: userInfo.contactNumber,
      },
    })
  } else if (userInfo.role === UserRoles.PASSENGER) {
    profileInfo = await prisma.passenger.findUnique({
      where: {
        contactNumber: userInfo.contactNumber,
      },
    })
  }

  return {
    ...userInfo,
    ...profileInfo,
  }
}

const updateMyProfile = async (user: IAuthUser, req: Request) => {
  const file = req.file
  const payload = req.body
  const userInfo = await prisma.user.findUniqueOrThrow({
    where: {
      contactNumber: user?.contactNumber,
      status: UserStatus.ACTIVE,
    },
  })

  let profileInfo

  if (user?.role === UserRoles.SUPER_ADMIN) {
    profileInfo = await prisma.superAdmin.update({
      where: {
        contactNumber: userInfo.contactNumber,
      },
      data: {
        ...payload,
        profileImage: file?.path,
      },
    })
  } else if (user?.role === UserRoles.OWNER) {
    profileInfo = await prisma.busCompanyOwner.update({
      where: {
        contactNumber: userInfo.contactNumber,
      },
      data: {
        ...payload,
        profileImage: file?.path,
      },
    })
  }

  return profileInfo
}

export const UserService = {
  createBusCompanyEmployeeIntoDB,
  createBusCounterEmployeeIntoDB,
  createOwnerIntoDB,
  getSpicificUser,
  getAllBusOwner,
  getAllPassenger,
  getAllEmployeeForSpicificBusCompany,
  getMe,
  updateMyProfile,
}
